/*
 *	colors.c
 */

#include "filer.h"

// #define COLORS_PATH	"\\system\\colors.ini"

static const u8	sTextColor[] = "Text color";
static const u8	sBackColor[] = "Back color";

static const u16	aDefaultColors[] = {
	RGB(0,0,0),
	RGB(31,31,31),
	RGB(0,0,31),
	RGB(0,31,0),
	RGB(0,31,31),
	RGB(31,0,0),
	RGB(31,0,31),
	RGB(31,31,0),
};

#define nDefaultColors	arraysizeof(aDefaultColors)

#define PALETTE_GAP	2
#define PALETTE_SIZE	4
#define PALETTE_NEXT	(PALETTE_GAP * 2 + PALETTE_SIZE - 1)

static u16		aColors[MAX_COLORS];
static int		nColors;

static u16		TextColor;
static u16		BackColor;
static int		TextIndex;
static int		BackIndex;

/*
 *	IsValidColors
 */
static BOOL IsValidColors(PSCRNCOLORS psc)
{
	if (memcmp((u8 *)psc->sig, SC_SIG, sizeof(psc->sig)) ||
		psc->ver != SC_VER ||
		psc->size != sizeof(*psc))
		return FALSE;
	return TRUE;
}

/*
 *	LoadColors
 */
void LoadColors()
{
	PSCRNCOLORS	psc;
	u16		fgcolor, bgcolor;

	psc = (PSCRNCOLORS)SCRNCOLORS_ADDR;
	if (!IsValidColors(psc)) {
		fgcolor = COLOR_WHITE;
		bgcolor = COLOR_BLACK;
	} else {
		fgcolor = ReadSRAM16(psc->fgcolor);
		bgcolor = ReadSRAM16(psc->bgcolor);
	}
	SetScrnColor(fgcolor, bgcolor);
}

/*
 *	SaveColors
 */
static void SaveColors()
{
	PSCRNCOLORS	psc;
	u16		fgcolor, bgcolor;

	psc = (PSCRNCOLORS)SCRNCOLORS_ADDR;
	if (!IsValidColors(psc)) {
		/*
		 *	Build ScrnColors
		 */
		WriteSRAMByte(psc->sig, SC_SIG, sizeof(psc->sig));
		WriteSRAM8(&psc->ver, SC_VER);
		WriteSRAM8(&psc->size, sizeof(*psc));
	}

	GetScrnColor(&fgcolor, &bgcolor);
	WriteSRAM16(psc->fgcolor, fgcolor);
	WriteSRAM16(psc->bgcolor, bgcolor);
}

/*
 *	SetPaletteIndex
 */
void SetPaletteIndex(u16 *pcolor, int *pindex, int index, u16 *ppal, int npal)
{
	int		i;

	for (i = 0; i < npal; i++) {
		if (*pcolor == ppal[i]) {
			*pindex = i;
			return;
		}
	}
	*pindex = index;
	*pcolor = ppal[index];
}

/*
 *	DrawColorPalette
 */
static void DrawColorPalette(PRECT prc, u16 *ppal, int npal)
{
	int		i;

	for (i = 0; i < npal; i++)
		FillScrn(prc->x + (i * PALETTE_NEXT) + PALETTE_GAP,
			prc->y + PALETTE_GAP, 
			PALETTE_SIZE,
			PALETTE_SIZE,
			ppal[i]);
}

/*
 *	ColorSelection
 */
int ColorSelection(u8 *title)
{
	CREATEWIN	cw;
	RECT		rc;
	RECT		rctc, rcbc;
	RECT		rcpv;
	int		hsmall;
	int		w, wt, wb;
	int		key;
	u16		*ppal;
	int		npal;
	u16		fgcolor, bgcolor;
	int		*pcurindex;
	u16		*pcurcolor;
	PRECT		pcurrc;
	PXFONTOBJ	pxf;

	/*
	 *	Put frame
	 */

	memset(&cw, 0, sizeof cw);
	CreateWinFrame(&cw);
	SetWinTitle(title);

	XfontSelectFontObject(FO_SMALL);
	hsmall = XfontGetCharHeight();
	pxf = XfontGetFontObject(0x81);

#if 0
	wt = XfontGetStrWidth((u8 *)sTextColor);
	wb = XfontGetStrWidth((u8 *)sBackColor);
	w = max(wt, wb);
	w = (w + 31) & ~31;
#endif
	w = XfontGetMaxCharWidth(' ') * 9;

	GetClientRect(&rc);
	LeftPrintf(rc.x, rc.y, w, "%s", sTextColor);
	LeftPrintf(rc.x, rc.y + hsmall, w, "%s", sBackColor);

	GetScrnColor(&fgcolor, &bgcolor);
	ppal = (u16 *)aColors;
	npal = nColors;
	TextColor = fgcolor;
	BackColor = bgcolor;

	rctc.x = rc.x + w;
	rctc.y = rc.y;
	rctc.w = rc.w - w;
	rctc.h = hsmall;
	DrawColorPalette(&rctc, ppal, npal);
	SetPaletteIndex(&TextColor, &TextIndex, 1, ppal, npal);

	rcbc.x = rctc.x;
	rcbc.y = rctc.y + hsmall;
	rcbc.w = rctc.w;
	rcbc.h = rctc.h;
	DrawColorPalette(&rcbc, ppal, npal);
	SetPaletteIndex(&BackColor, &BackIndex, 0, ppal, npal);

	rcpv.x = rc.x;
	rcpv.y = rcbc.y + (hsmall * 4);
	rcpv.w = w;
	rcpv.h = rcbc.h * 2;

	/*
	 *	Select item
	 */

	pcurindex = &TextIndex;
	pcurcolor = &TextColor;
	pcurrc = &rctc;

	for (;;) {
		BoxScrn(pcurrc->x + (*pcurindex) * PALETTE_NEXT,
			pcurrc->y,
			PALETTE_NEXT + 1,
			PALETTE_NEXT + 1,
			fgcolor);

		SetScrnColor(TextColor, BackColor);
		LeftPrintf(rcpv.x, rcpv.y, rcpv.w, "%s", sPreviewSample);
		if (pxf)
			LeftPrintf(rcpv.x, rcpv.y + hsmall, rcpv.w, "%s",
				sPreviewKanjiSample);
		SetScrnColor(fgcolor, bgcolor);

		while (!(key = GetKey()))
			;

		BoxScrn(pcurrc->x + (*pcurindex) * PALETTE_NEXT,
			pcurrc->y,
			PALETTE_NEXT + 1,
			PALETTE_NEXT + 1,
			bgcolor);

		switch (GetKeyFunc(key)) {
		case KF_OPTIONS:
		case KF_BACK:
			return LB_CANCEL;
		case KF_OK:
			if (TextColor != BackColor)
				goto out;
			break;
		case KF_PREVITEM:
		case KF_NEXTITEM:
			if (pcurindex == &TextIndex) {
				pcurindex = &BackIndex;
				pcurcolor = &BackColor;
				pcurrc = &rcbc;
			} else {
				pcurindex = &TextIndex;
				pcurcolor = &TextColor;
				pcurrc = &rctc;
			}
			break;
		case KF_PREVPAGE:
			if (*pcurindex > 0)
				(*pcurindex)--;
			else
				*pcurindex = npal - 1;
			*pcurcolor = ppal[*pcurindex];
			break;
		case KF_NEXTPAGE:
			if (*pcurindex < npal - 1)
				(*pcurindex)++;
			else
				*pcurindex = 0;
			*pcurcolor = ppal[*pcurindex];
			break;
		}
	}

out:
	/*
	 *	Set color
	 */

	SetScrnColor(TextColor, BackColor);
	SaveColors();

	return 0;
}

/*
 *	LoadCustomColors
 */
int LoadCustomColors(PTOKEN ptok)
{
	int		t;
	int		col;
	BOOL		customized;

	customized = FALSE;
	while ((t = GetToken(ptok)) != T_EOF) switch (t) {
	default:
		t = T_ERROR;
		goto out;
	case '\n':
		goto out;
	case T_VALUE:
		col = (int)ptok->pvalue;
		if (!customized) {
			nColors = 2;
			customized = TRUE;
		}
		if (nColors >= MAX_COLORS)
			goto out;
		aColors[nColors++] = col;
		break;
	}

out:
	return t;
}

/*
 *	InitColor
 */
void InitColor()
{
	memcpy(aColors, aDefaultColors, sizeof aDefaultColors);
	nColors = nDefaultColors;
}

