/*
 *	ini.c
 */

#include "filer.h"

#define DEBUG

#define INIFILE_PATH	"\\system\\gbafiler.ini"

extern const u8	*aError[];
extern int	nError;

/*
 *	ErrorIniFile
 */
void ErrorIniFile(PTOKEN ptok, int ecode, u8 *popt)
{
	ClearScrn();
	if (ecode < 0 || ecode >= nError)
		ecode = 0;

	if (popt)
		Fix8printf(0, 0, "%s: %s in %d",
			popt, aError[ecode], ptok->curline);
	else
		Fix8printf(0, 0, "%s in %d", aError[ecode], ptok->curline);

	while (GetKey() == 0) ;
}

const VALTABLE	aVTGeneral[] = {
	{ "Colors",	LoadCustomColors },
	{ NULL, NULL },
};

#define nVTGeneral	arraysizeof(aVTGeneral)

extern const VALTABLE	aVTKeyMap[];

const KEYTABLE	aKeyTable[] = {
	{ "General",	(PVALTABLE)aVTGeneral, },
	{ "KeyMap",	(PVALTABLE)aVTKeyMap, },
};

#define nKeyTable	((int)arraysizeof(aKeyTable))

/*
 *	LoadIniFile
 */
void LoadIniFile()
{
	PDIRENT		pdir;
	PTOKEN		ptok;
	u8		*ptop;
	u8		*pend;
	int		t;
	u8		*pname;
	int		len;
	TOKEN		tok;
	int		i;
	PKEYTABLE	pkey;
	PVALTABLE	pval;

	if (!(pdir = FsGetDirEnt(INIFILE_PATH)) || (pdir->attr & DA_DIR))
		return;

	ptok = &tok;
	ptop = pdir->fileaddr + pdir->namesize;
	pend = &ptop[pdir->filesize];
	InitToken(ptok, ptop, pend);

	pkey = NULL;

	while ((t = GetToken(ptok)) != T_EOF) {
		switch (t) {
		default:
			ErrorIniFile(ptok, E_SYNTAX, NULL);
			return;
		case '\n':
			break;
		case '[':
			if ((t = GetToken(ptok)) != T_NAME) {
				ErrorIniFile(ptok, E_SYNTAX, NULL);
				return;
			}
			pname = ptok->pvalue;
			len = ptok->len;
			if ((t = GetToken(ptok)) != ']') {
				ErrorIniFile(ptok, E_SYNTAX, NULL);
				return;
			}

			for (i = 0; i < nKeyTable; i++) {
				if (_strnicmp(pname, aKeyTable[i].name, len)
					== 0)
				{
					pkey = (PKEYTABLE)&aKeyTable[i];
					break;
				}
			}
			if (i >= nKeyTable) {
				ErrorIniFile(ptok, E_UNKNOWN, NULL);
				return;
			}
			break;
		case T_NAME:
			if (!pkey) {
				ErrorIniFile(ptok, E_SYNTAX, NULL);
				return;
			}
			pname = ptok->pvalue;
			len = ptok->len;
			if ((t = GetToken(ptok)) != '=') {
				ErrorIniFile(ptok, E_SYNTAX, NULL);
				return;
			}

			pval = NULL;
			for (i = 0, pval = pkey->pval; pval->name; i++, pval++)
			{
				if (_strnicmp(pname, pval->name, len) == 0)
					break;
			}
			if (!pval->name) {
				ErrorIniFile(ptok, E_UNKNOWN, NULL);
				return;
			}

			switch (pval->phandler(ptok)) {
			case T_EOF:
				return;
			case T_ERROR:
				ErrorIniFile(ptok, E_SYNTAX, NULL);
				return;
			}
			break;
		}
	}
}

