/*
 *	screen.c
 */

#include "filer.h"

#define PREVIEW_WANDH	40

/*
 *	IsValidScrnDir
 */
static BOOL IsValidScrnDir(PSCRNDIR psd)
{
	if (memcmp((u8 *)psd->sig, SD_SIG, sizeof(psd->sig)) ||
		psd->ver != SD_VER ||
		psd->size != sizeof(*psd))
		return FALSE;
	return TRUE;
}

/*
 *	LoadScrnDir
 */
static void LoadScrnDir()
{
	PSCRNDIR	psd;
	u8		mode;

	psd = (PSCRNDIR)SCRNDIR_ADDR;
	if (!IsValidScrnDir(psd))
		mode = SM_HORZ;
	else
		mode = ReadSRAM8(&psd->mode);
	SetWinMode(mode);
}

/*
 *	SaveScrnDir
 */
static void SaveScrnDir()
{
	PSCRNDIR	psd;
	u8		mode;

	psd = (PSCRNDIR)SCRNDIR_ADDR;
	if (!IsValidScrnDir(psd)) {
		/*
		 *	Build ScrnDir
		 */
		WriteSRAMByte(psd->sig, SD_SIG, sizeof(psd->sig));
		WriteSRAM8(&psd->ver, SD_VER);
		WriteSRAM8(&psd->size, sizeof(*psd));
	}

	mode = GetScrnMode();
	WriteSRAM8(&psd->mode, mode);
}

/*
 *	IndexToScreen
 */
static int IndexToScreen(int index)
{
	switch (index) {
	default:
		return -1;
	case 0:
		return SM_HORZ;
	case 1:
		return SM_VERTRIGHT;
	case 2:
		return SM_VERTLEFT;
	}
}

/*
 *	GetPreviewRect
 */
static void GetPreviewRect(PRECT prc)
{
	PSCRNOBJ	pso;

	pso = GetScrnObject();
	prc->x = (GetScrnWidth(pso) - PREVIEW_WANDH) / 2;
	prc->y = (GetScrnHeight(pso) - PREVIEW_WANDH) / 2;
	prc->w = PREVIEW_WANDH;
	prc->h = PREVIEW_WANDH;
}

/*
 *	PreviewScreen
 */
static int PreviewScreen(int index, void *params)
{
	int		oldmode;
	int		mode;
	int		h;
	RECT		rc;
	PXFONTOBJ	pxf;

	mode = IndexToScreen(index);
	if (mode < 0)
		return -1;

	GetPreviewRect(&rc);
	EraseScrn(rc.x, rc.y, rc.w, rc.h);

	oldmode = SetScrnMode(mode);

	GetPreviewRect(&rc);
	XfontSelectFontObject(FO_SMALL);
	h = XfontGetCharHeight();
	pxf = XfontGetFontObject(0x81);
	LeftPrintf(rc.x, rc.y, rc.w, "%s", sPreviewSample);
	if (pxf)
		LeftPrintf(rc.x, rc.y + h, rc.w, "%s", sPreviewKanjiSample);

	SetScrnMode(oldmode);
}

/*
 *	SelectScreen
 */
int SelectScreen(u8 *title)
{
	int		i;
	LISTBOX		lb;
	u8		*items[3];
	RECT		rc;
	int		curmode;
	int		mode;
	int		h;

	curmode = GetScrnMode();

	memset(&lb, 0, sizeof lb);
	lb.title = title;
	items[0] = "Horizontal";
	items[1] = "Vertical (Right to upper)";
	items[2] = "Vertical (Left to upper)";
	lb.items = items;
	lb.nitems = arraysizeof(items);
	lb.defitem = curmode;

	h = XfontGetCharHeight();
	GetClientRect(&rc);
	lb.rc.x = rc.x;
	lb.rc.y = rc.y;
	lb.rc.w = rc.w;
	lb.rc.h = h * lb.nitems;

	lb.callback = PreviewScreen;

	i = ListBox(&lb);

	if (i < 0)
		return i;
	if (i == curmode)
		return LB_CANCEL;

	mode = IndexToScreen(i);
	SetWinMode(mode);
	SaveScrnDir();

	return i;
}

/*
 *	InitScreen
 */
void InitScreen()
{
	ScrnInit();
	LoadScrnDir();
}

