/*
 *	subr.c
 */

#include <stdarg.h>
#include "filer.h"

#define KEY_REPEAT_TIME	500
#define KEY_WAIT	TIMER_INTMS

static u16	LastKey;
static u32	WaitTimer;
static u32	RepeatTimer;

/*
 *	GetKey
 */
int GetKey()
{
	u16		key;

	if (ClockTick - WaitTimer < KEY_WAIT)
		return 0;

	key = (~READ_REG16(KEY_STS)) & KEY_ALL;
	WaitTimer = ClockTick;

	if (key && key == LastKey) {
		if (ClockTick - RepeatTimer < KEY_REPEAT_TIME)
			return 0;
		return KEY_REPEAT | key;
	}

	LastKey = key;
	RepeatTimer = ClockTick;

	return key;
}

/*
 *	InitKey
 */
void InitKey()
{
	LastKey = 0;
	WaitTimer = RepeatTimer = ClockTick;
}

/*
 *	PutStr
 */
int PutStr(int x, int y, u8 *s)
{
	u8		c;
	int		x0;

	x0 = x;
	while (*s)
		x += XfontPutChar(x, y, &s);

	return x - x0;
}

/*
 *	Printf
 */
int Printf(int x, int y, u8 *fmt, ...)
{
	va_list		vp;
	int		n;
	u8		buf[128];

	va_start(vp, fmt);
	_vsprintf(buf, fmt, vp);
	n = PutStr(x, y, buf);
	va_end(vp);

	return n;
}

/*
 *	LeftPrintf
 */
int LeftPrintf(int x, int y, int w, u8 *fmt, ...)
{
	va_list		vp;
	int		n;
	int		thresh;
	u8		c;
	u8		*bp;
	u8		*bp1;
	u8		buf[256];

	va_start(vp, fmt);
	_vsprintf(buf, fmt, vp);

	n = 0;
	thresh = XfontGetMaxCharWidth('.') * 3;
	bp = buf;
	bp1 = NULL;
	while (*bp) {
		n += XfontGetCharWidth(&bp);
		if ((n + thresh) >= w && !bp1)
			bp1 = bp;
	}
	if (n > w && bp1)
		strcpy(bp1, "...");

	n = PutStr(x, y, buf);
	if (n < w)
		EraseScrn(x + n, y, w - n, XfontGetCharHeight());
	va_end(vp);

	return x;
}

/*
 *	RightPrintf
 */
int RightPrintf(int x, int y, int w, u8 *fmt, ...)
{
	va_list		vp;
	int		x0;
	u8		c;
	u8		*bp;
	u8		buf[256];

	va_start(vp, fmt);
	_vsprintf(buf, fmt, vp);

	x0 = x + w;
	bp = buf;
	while (*bp && x0 > x)
		x0 -= XfontGetCharWidth(&bp);
	*bp = '\0';

	if (x0 > x)
		EraseScrn(x, y, x0 - x, XfontGetCharHeight());
	PutStr(x0, y, buf);
	va_end(vp);

	return x;
}

/*
 *	_strnicmp
 */
int _strnicmp(u8 *ps, u8 *pd, int len)
{
	while (len-- > 0) {
		if (toupper(*ps) != toupper(*pd))
			return *ps - *pd;
		ps++, pd++;
	}
	return *pd;
}

