/*
 *	viewbmp.c
 */

#include "filer.h"

#define X_ERROR		80
#define Y_ERROR		20

#define MAKEU32(w)	((u32)(w[0] | (w[1] << 16)))
#define RGB16(r, g, b)	\
	((((b) << 7) & 0x7c00) | (((g) << 2) & 0x3e0) | (((r) >> 3) & 0x1f))

/*
 *	IsBitmap
 */
static BOOL IsBitmap(u8 *pfile)
{
	return memcmp(pfile, BMFH_SIG, 2) == 0;
}

/*
 *	ShowBMP
 */
static void ShowBMP(int x, int y, u8 *pbuf)
{
	PBMPFILEHDR	pbmf;
	PBMPINFOHDR	pbmi;
	u8		*bp0, *bp;
	vu16		*vp, *vp0;
	u8		r, g, b;
	int		width, height;
	int		wsize;
	PSCRNOBJ	pso;
	int		sw, sh;
	int		xd, yd;
	u16		fgcolor, bgcolor;

	if (!IsBitmap(pbuf))
		return;

	pso = GetScrnObject();
	sw = GetScrnWidth(pso);
	sh = GetScrnHeight(pso);
	xd = GetScrnNextX(pso);
	yd = GetScrnNextY(pso);
	GetScrnColor(&fgcolor, &bgcolor);

	pbmf = (PBMPFILEHDR)pbuf;
	pbmi = (PBMPINFOHDR)(pbuf + SIZEOF_BMPFILEHDR);
	width = MAKEU32(pbmi->biWidth) - x;
	if (width <= 0)
		return;
	height = MAKEU32(pbmi->biHeight) - y;
	if (height <= 0)
		return;

	width = min(width, sw);
	height = min(height, sh);
	wsize = (MAKEU32(pbmi->biWidth) * 3 + 3) & ~3;

	bp0 = &pbuf[MAKEU32(pbmf->bfSize) - y * wsize + x * 3];
	vp = pso->ScrnPosToVram(0, 0);

	for (y = 0; y < height; y++) {
		vp0 = vp;
		bp0 -= wsize;
		bp = bp0;
		for (x = 0; x < width; x++) {
			b = *bp++;
			g = *bp++;
			r = *bp++;
			*vp = RGB16(r, g, b);
			vp += xd;
		}
		while (x++ < sw) {
			*vp = bgcolor;
			vp += xd;
		}
		vp = vp0 + yd;
	}
	if (y < sh)
		EraseScrn(0, y, sw, sh - y);
}

/*
 *	ViewBMP
 */
BOOL ViewBMP(PDIRENT pdir)
{
	PBMPFILEHDR	pbmf;
	PBMPINFOHDR	pbmi;
	int		key;
	u32		wsize, hsize;
	int		width, height;
	int		curx, cury;
	int		dif;
	PSCRNOBJ	pso;
	int		sw, sh;

	if (!pdir->filesize)
		return FALSE;

	pbmf = (PBMPFILEHDR)(pdir->fileaddr + pdir->namesize);
	if (!IsBitmap((u8 *)pbmf) || MAKEU32(pbmf->bfSize) != pdir->filesize)
		return FALSE;

	pbmi = (PBMPINFOHDR)((u8 *)pbmf + SIZEOF_BMPFILEHDR);
	if (MAKEU32(pbmi->biSize) != sizeof(*pbmi))
		return FALSE;

	if (pbmi->biBitCount != 24)
		// TODO: Support another color modes.
		return FALSE;

	pso = GetScrnObject();
	sw = GetScrnWidth(pso);
	sh = GetScrnHeight(pso);

	width = MAKEU32(pbmi->biWidth);
	wsize = (width * 3 + 3) & ~3;	// round up to 4 bytes
	hsize = MAKEU32(pbmi->biHeight);
	height = hsize;

	curx = cury = 0;

	ShowBMP(curx, cury, (u8 *)pbmf);

	for (;;) {
		while (!(key = GetKey())) ;
		dif = (key & KEY_REPEAT) ? 32 : 8;

		switch (GetKeyFunc(key)) {
		case KF_BACK:
			goto out;
		case KF_PREVPAGE:
			if ((curx - dif) >= 0) {
				curx -= dif;
				ShowBMP(curx, cury, (u8 *)pbmf);
			}
			break;
		case KF_NEXTPAGE:
			if ((curx + sw + dif) < width) {
				curx += dif;
				ShowBMP(curx, cury, (u8 *)pbmf);
			}
			break;
		case KF_PREVITEM:
			if ((cury - dif) >= 0) {
				cury -= dif;
				ShowBMP(curx, cury, (u8 *)pbmf);
			}
			break;
		case KF_NEXTITEM:
			if ((cury + sh + dif) < height) {
				cury += dif;
				ShowBMP(curx, cury, (u8 *)pbmf);
			}
			break;
		}
	}

out:
	return TRUE;
}
