/*
 *	xfont.c
 */

#include <xfont.h>
#include <scrn.h>

static const u8	BitMasks[] = { 0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01 };

static u8	FontPitch;

/*
 *	XfontSetFontPitch
 */
int XfontSetFontPitch(int pitch)
{
	int		opitch;

	opitch = FontPitch;
	FontPitch = pitch;

	return opitch;
}

/*
 *	IsValidXfont
 */
BOOL IsValidXfont(PXFONTHEAD pxf)
{
	return memcmp(pxf->sig, XFONT_SIG, sizeof(pxf->sig)) == 0 &&
		pxf->ver == XFONT_VER &&
		pxf->size == sizeof(*pxf);
}

/*
 *	iwXfontGetFontName
 */
static
attriwram
u8 *iwXfontGetFontName()
{
	return XfontGetFontObject(' ')->pxf->name;
}

/*
 *	iwXfontGetMaxCharWidth
 */
static
attriwram
int iwXfontGetMaxCharWidth(u8 code)
{
	PXFONTHEAD	pxf;

	pxf = XfontGetFontObject(code)->pxf;
	return pxf->width + pxf->xgap;
}

/*
 *	iwXfontGetCharHeight
 */
static
attriwram
int iwXfontGetCharHeight()
{
	PXFONTHEAD	pxf;
	u8		pitch;

	pxf = XfontGetFontObject(' ')->pxf;
	pitch = (FontPitch == FP_DEFAULT) ? pxf->ygap : FontPitch;
	return pxf->height + pitch;
}

/*
 *	iwXfontGetCharWidth
 */
static
attriwram
int iwXfontGetCharWidth(u8 **ppcode)
{
	PXFONTOBJ	pxo;
	PXFONTHEAD	pxf;
	int		off;
	u8		*pf;
	u16		code;

	pxo = XfontGetFontObject(**ppcode);
	pxf = pxo->pxf;
	code = pxo->getcode(ppcode);
	if (pxf->flags & XFF_PROP) {
		off = pxo->code2index(pxo, code) * pxf->unit;
		pf = ((u8 *)pxf) + pxf->size + off;

		return *pf + pxf->xgap;
	} else
		return pxf->width + pxf->xgap;
}

/*
 *	iwXfontGetStrWidth
 */
static
attriwram
int iwXfontGetStrWidth(u8 *pcode)
{
	int		w;

	w = 0;
	while (*pcode)
		w += iwXfontGetCharWidth(&pcode);

	return w;
}

/*
 *	iwXfontPutChar
 */
static
attriwram
int iwXfontPutChar(int x, int y, u8 **ppcode)
{
	PXFONTOBJ	pxo;
	PXFONTHEAD	pxf;
	vu16		*v, *v0;
	u8		*pf;
	int		off;
	int		i, j;
	int		width;
	u8		bmask;
	u8		fd;
	u16		code;
	PSCRNOBJ	pso;
	u16		fgcolor, bgcolor;
	int		xd, yd;
	u8		pitch;

	/*
	 *	Get font resources
	 */

	pxo = XfontGetFontObject(**ppcode);
	pxf = pxo->pxf;
	code = pxo->getcode(ppcode);
	if (pxf->flags & XFF_PROP) {
		off = pxo->code2index(pxo, code) * pxf->unit;
		pf = ((u8 *)pxf) + pxf->size + off;
		width = *pf++;
		bmask = 0x80;
	} else {
		off = pxo->code2index(pxo, code) * pxf->unit;
		pf = ((u8 *)pxf) + pxf->size + (off / NBPB);
		width = pxf->width;
		bmask = BitMasks[off % NBPB];
	}

	/*
	 *	Draw font
	 */

	pso = iwGetScrnObject();
	v = pso->ScrnPosToVram(x, y);
	xd = GetScrnNextX(pso);
	yd = GetScrnNextY(pso);
	iwGetScrnColor(&fgcolor, &bgcolor);

	fd = *pf++;
	for (i = 0; i < pxf->height; i++) {
		v0 = v;
		for (j = 0; j < width; j++) {
			*v = (fd & bmask) ? fgcolor : bgcolor;
			v += xd;
			if ((bmask >>= 1) == 0) {
				bmask = 0x80;
				fd = *pf++;
			}
		}
		for (j = 0; j < pxf->xgap; j++) {
			*v = bgcolor;
			v += xd;
		}
		v = v0 + yd;
	}

	pitch = (FontPitch == FP_DEFAULT) ? pxf->ygap : FontPitch;
	for (i = 0; i < pitch; i++) {
		v0 = v;
		for (j = 0; j < width + pxf->xgap; j++) {
			*v = bgcolor;
			v += xd;
		}
		v = v0 + yd;
	}

	return width + pxf->xgap;
}

u8 *(*XfontGetFontName)();
int (*XfontGetMaxCharWidth)(u8 code);
int (*XfontGetCharHeight)();
int (*XfontGetCharWidth)(u8 **ppcode);
int (*XfontGetStrWidth)(u8 *pcode);
int (*XfontPutChar)(int x, int y, u8 **ppcode);
PXFONTOBJ (*XfontGetFontObject)(u8 code);

extern attriwram PXFONTOBJ iwXfontGetFontObject(u8 code);

/*
 *	XfontInit
 */
void XfontInit()
{
	XfontInitCCT();

	XfontGetFontName = iwXfontGetFontName;
	XfontGetMaxCharWidth = iwXfontGetMaxCharWidth;
	XfontGetCharHeight = iwXfontGetCharHeight;
	XfontGetCharWidth = iwXfontGetCharWidth;
	XfontGetStrWidth = iwXfontGetStrWidth;
	XfontPutChar = iwXfontPutChar;
	XfontGetFontObject = iwXfontGetFontObject;

	XfontSetFontPitch(FP_DEFAULT);
}

